@@ -5,37 +5,14 @@ AWS client-side authentication: standard credentials providers and signing.
 # Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 # SPDX-License-Identifier: Apache-2.0.
 
-from __future__ import absolute_import
 import _awscrt
-from awscrt import isinstance_str, NativeResource
+from awscrt import NativeResource
 import awscrt.exceptions
 from awscrt.http import HttpRequest
 from awscrt.io import ClientBootstrap
 from concurrent.futures import Future
 import datetime
 from enum import IntEnum
-import time
-
-try:
-    _utc = datetime.timezone.utc
-except AttributeError:
-    # Python 2 lacks the datetime.timestamp() method.
-    # We can do the timestamp math ourselves, but only if datetime.tzinfo is set.
-    # Python 2 also lacks any predefined tzinfo classes (ex: datetime.timezone.utc),
-    # so we must define our own.
-    class _UTC(datetime.tzinfo):
-        ZERO = datetime.timedelta(0)
-
-        def utcoffset(self, dt):
-            return _UTC.ZERO
-
-        def tzname(self, dt):
-            return "UTC"
-
-        def dst(self, dt):
-            return _UTC.ZERO
-
-    _utc = _UTC()
 
 
 class AwsCredentials(NativeResource):
@@ -57,11 +34,11 @@ class AwsCredentials(NativeResource):
     __slots__ = ()
 
     def __init__(self, access_key_id, secret_access_key, session_token=None):
-        assert isinstance_str(access_key_id)
-        assert isinstance_str(secret_access_key)
-        assert isinstance_str(session_token) or session_token is None
+        assert isinstance(access_key_id, str)
+        assert isinstance(secret_access_key, str)
+        assert isinstance(session_token, str) or session_token is None
 
-        super(AwsCredentials, self).__init__()
+        super().__init__()
         self._binding = _awscrt.credentials_new(access_key_id, secret_access_key, session_token)
 
     @classmethod
@@ -98,7 +75,7 @@ class AwsCredentialsProviderBase(NativeResource):
     __slots__ = ()
 
     def __init__(self, binding=None):
-        super(AwsCredentialsProviderBase, self).__init__()
+        super().__init__()
 
         if binding is None:
             # TODO: create binding type that lets native code call into python subclass
@@ -156,9 +133,9 @@ class AwsCredentialsProvider(AwsCredentialsProviderBase):
         Returns:
             AwsCredentialsProvider:
         """
-        assert isinstance_str(access_key_id)
-        assert isinstance_str(secret_access_key)
-        assert isinstance_str(session_token) or session_token is None
+        assert isinstance(access_key_id, str)
+        assert isinstance(secret_access_key, str)
+        assert isinstance(session_token, str) or session_token is None
 
         binding = _awscrt.credentials_provider_new_static(access_key_id, secret_access_key, session_token)
         return cls(binding)
@@ -208,24 +185,32 @@ class AwsSignatureType(IntEnum):
     """
 
 
-class AwsSignedBodyValueType(IntEnum):
-    """Controls what goes in the canonical request's body value."""
+class AwsSignedBodyValue:
+    """
+    Values for use with :attr:`AwsSigningConfig.signed_body_value`.
 
-    EMPTY = 0
-    """Use the SHA-256 of the empty string."""
+    Some services use special values (e.g. "UNSIGNED-PAYLOAD") when the body
+    is not being signed in the usual way.
+    """
 
-    PAYLOAD = 1
-    """Use the SHA-256 of the actual payload."""
+    EMPTY_SHA256 = 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855'
+    """The SHA-256 of the empty string."""
 
-    UNSIGNED_PAYLOAD = 2
-    """Use the literal string "UNSIGNED-PAYLOAD"."""
+    UNSIGNED_PAYLOAD = 'UNSIGNED-PAYLOAD'
+    """Unsigned payload option (not accepted by all services)"""
+
+    STREAMING_AWS4_HMAC_SHA256_PAYLOAD = 'STREAMING-AWS4-HMAC-SHA256-PAYLOAD'
+    """Each payload chunk will be signed (not accepted by all services)"""
+
+    STREAMING_AWS4_HMAC_SHA256_EVENTS = 'STREAMING-AWS4-HMAC-SHA256-EVENTS'
+    """Each event will be signed (not accepted by all services)"""
 
 
 class AwsSignedBodyHeaderType(IntEnum):
     """
     Controls if signing adds a header containing the canonical request's signed body value.
 
-    See :class:`AwsSignedBodyValueType`.
+    See :attr:`AwsSigningConfig.signed_body_value`.
     """
 
     NONE = 0
@@ -280,9 +265,13 @@ class AwsSigningConfig(NativeResource):
         should_normalize_uri_path (bool): Whether the resource paths are
             normalized when building the canonical request.
 
-        signed_body_value_type (AwsSignedBodyValueType): Controls what goes in
-            the canonical request's body value. Default is to use the SHA-256
-            of the actual payload.
+        signed_body_value (Optional[str]): If set, this value is used as the
+            canonical request's body value. Typically, this is the SHA-256
+            of the payload, written as lowercase hex. If this has been
+            precalculated, it can be set here. Special values used by certain
+            services can also be set (see :class:`AwsSignedBodyValue`). If `None`
+            is passed (the default), the typical value will be calculated from
+            the payload during signing.
 
         signed_body_header_type (AwsSignedBodyHeaderType): Controls if signing
             adds a header containing the canonical request's signed body value.
@@ -308,7 +297,7 @@ class AwsSigningConfig(NativeResource):
         'should_sign_header',
         'use_double_uri_encode',
         'should_normalize_uri_path',
-        'signed_body_value_type',
+        'signed_body_value',
         'signed_body_header_type',
         'expiration_in_seconds',
         'omit_session_token',
@@ -324,7 +313,7 @@ class AwsSigningConfig(NativeResource):
                  should_sign_header=None,
                  use_double_uri_encode=True,
                  should_normalize_uri_path=True,
-                 signed_body_value_type=AwsSignedBodyValueType.PAYLOAD,
+                 signed_body_value=None,
                  signed_body_header_type=AwsSignedBodyHeaderType.NONE,
                  expiration_in_seconds=None,
                  omit_session_token=False,
@@ -333,28 +322,19 @@ class AwsSigningConfig(NativeResource):
         assert isinstance(algorithm, AwsSigningAlgorithm)
         assert isinstance(signature_type, AwsSignatureType)
         assert isinstance(credentials_provider, AwsCredentialsProviderBase)
-        assert isinstance_str(region)
-        assert isinstance_str(service)
+        assert isinstance(region, str)
+        assert isinstance(service, str)
         assert callable(should_sign_header) or should_sign_header is None
-        assert isinstance(signed_body_value_type, AwsSignedBodyValueType)
+        assert signed_body_value is None or (isinstance(signed_body_value, str) and len(signed_body_value) > 0)
         assert isinstance(signed_body_header_type, AwsSignedBodyHeaderType)
         assert expiration_in_seconds is None or expiration_in_seconds > 0
 
-        super(AwsSigningConfig, self).__init__()
+        super().__init__()
 
         if date is None:
-            date = datetime.datetime.now(_utc)
+            date = datetime.datetime.now(datetime.timezone.utc)
 
-        try:
-            timestamp = date.timestamp()
-        except AttributeError:
-            # Python 2 doesn't have datetime.timestamp() function.
-            # If it did we could just call it from binding code instead of calculating it here.
-            if date.tzinfo is None:
-                timestamp = time.mktime(date.timetuple())
-            else:
-                epoch = datetime.datetime(1970, 1, 1, tzinfo=_utc)
-                timestamp = (date - epoch).total_seconds()
+        timestamp = date.timestamp()
 
         self._priv_should_sign_cb = should_sign_header
 
@@ -379,7 +359,7 @@ class AwsSigningConfig(NativeResource):
             should_sign_header_wrapper,
             use_double_uri_encode,
             should_normalize_uri_path,
-            signed_body_value_type,
+            signed_body_value,
             signed_body_header_type,
             expiration_in_seconds,
             omit_session_token)
@@ -463,11 +443,16 @@ class AwsSigningConfig(NativeResource):
         return _awscrt.signing_config_get_should_normalize_uri_path(self._binding)
 
     @property
-    def signed_body_value_type(self):
+    def signed_body_value(self):
         """
-        AwsSignedBodyValueType: Controls what goes in the canonical request's body value.
+        Optional[str]: What to use as the canonical request's body value.
+        If `None` is set (the default), a value will be calculated from
+        the payload during signing. Typically, this is the SHA-256 of the
+        payload, written as lowercase hex. If this has been precalculated,
+        it can be set here. Special values used by certain services can also
+        be set (see :class:`AwsSignedBodyValue`).
         """
-        return AwsSignedBodyValueType(_awscrt.signing_config_get_signed_body_value_type(self._binding))
+        return _awscrt.signing_config_get_signed_body_value(self._binding)
 
     @property
     def signed_body_header_type(self):
