@@ -7,10 +7,9 @@ All network operations in `awscrt.http` are asynchronous.
 # Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 # SPDX-License-Identifier: Apache-2.0.
 
-from __future__ import absolute_import
 import _awscrt
 from concurrent.futures import Future
-from awscrt import NativeResource, isinstance_str
+from awscrt import NativeResource
 import awscrt.exceptions
 from awscrt.io import ClientBootstrap, EventLoopGroup, DefaultHostResolver, InputStream, TlsConnectionOptions, SocketOptions
 from enum import IntEnum
@@ -30,7 +29,7 @@ class HttpConnectionBase(NativeResource):
     __slots__ = ('_shutdown_future', '_version')
 
     def __init__(self):
-        super(HttpConnectionBase, self).__init__()
+        super().__init__()
 
         self._shutdown_future = Future()
 
@@ -113,7 +112,7 @@ class HttpClientConnection(HttpConnectionBase):
             Otherwise, it will contain an exception.
         """
         assert isinstance(bootstrap, ClientBootstrap) or bootstrap is None
-        assert isinstance_str(host_name)
+        assert isinstance(host_name, str)
         assert isinstance(port, int)
         assert isinstance(tls_connection_options, TlsConnectionOptions) or tls_connection_options is None
         assert isinstance(socket_options, SocketOptions) or socket_options is None
@@ -179,7 +178,7 @@ class HttpClientConnection(HttpConnectionBase):
     def request(self, request, on_response=None, on_body=None):
         """Create :class:`HttpClientStream` to carry out the request/response exchange.
 
-        NOTE: The stream sends no data until :meth:`HttpClientStream.activate()`
+        NOTE: The HTTP stream sends no data until :meth:`HttpClientStream.activate()`
         is called. Call activate() when you're ready for callbacks and events to fire.
 
         Args:
@@ -188,7 +187,7 @@ class HttpClientConnection(HttpConnectionBase):
             on_response: Optional callback invoked once main response headers are received.
                 The function should take the following arguments and return nothing:
 
-                *   `http_stream` (:class:`HttpClientStream`): Stream carrying
+                *   `http_stream` (:class:`HttpClientStream`): HTTP stream carrying
                     out this request/response exchange.
 
                 *   `status_code` (int): Response status code.
@@ -198,13 +197,13 @@ class HttpClientConnection(HttpConnectionBase):
 
                 *   `**kwargs` (dict): Forward compatibility kwargs.
 
-                An exception raise by this function will cause the stream to end in error.
+                An exception raise by this function will cause the HTTP stream to end in error.
                 This callback is always invoked on the connection's event-loop thread.
 
             on_body: Optional callback invoked 0+ times as response body data is received.
                 The function should take the following arguments and return nothing:
 
-                *   `http_stream` (:class:`HttpClientStream`): Stream carrying
+                *   `http_stream` (:class:`HttpClientStream`): HTTP stream carrying
                     out this request/response exchange.
 
                 *   `chunk` (buffer): Response body data (not necessarily
@@ -212,7 +211,7 @@ class HttpClientConnection(HttpConnectionBase):
 
                 *   `**kwargs` (dict): Forward-compatibility kwargs.
 
-                An exception raise by this function will cause the stream to end in error.
+                An exception raise by this function will cause the HTTP stream to end in error.
                 This callback is always invoked on the connection's event-loop thread.
 
         Returns:
@@ -226,7 +225,7 @@ class HttpStreamBase(NativeResource):
     __slots__ = ('_connection', '_completion_future', '_on_body_cb')
 
     def __init__(self, connection, on_body=None):
-        super(HttpStreamBase, self).__init__()
+        super().__init__()
         self._connection = connection
         self._completion_future = Future()
         self._on_body_cb = on_body
@@ -245,11 +244,11 @@ class HttpStreamBase(NativeResource):
 
 
 class HttpClientStream(HttpStreamBase):
-    """Stream that sends a request and receives a response.
+    """HTTP stream that sends a request and receives a response.
 
     Create an HttpClientStream with :meth:`HttpClientConnection.request()`.
 
-    NOTE: The stream sends no data until :meth:`HttpClientStream.activate()`
+    NOTE: The HTTP stream sends no data until :meth:`HttpClientStream.activate()`
     is called. Call activate() when you're ready for callbacks and events to fire.
 
     Attributes:
@@ -268,7 +267,7 @@ class HttpClientStream(HttpStreamBase):
         assert callable(on_response) or on_response is None
         assert callable(on_body) or on_body is None
 
-        super(HttpClientStream, self).__init__(connection, on_body)
+        super().__init__(connection, on_body)
 
         self._on_response_cb = on_response
         self._response_status_code = None
@@ -288,7 +287,7 @@ class HttpClientStream(HttpStreamBase):
     def activate(self):
         """Begin sending the request.
 
-        The stream does nothing until this is called. Call activate() when you
+        The HTTP stream does nothing until this is called. Call activate() when you
         are ready for its callbacks and events to fire.
         """
         _awscrt.http_client_stream_activate(self)
@@ -318,7 +317,7 @@ class HttpMessageBase(NativeResource):
     def __init__(self, binding, headers, body_stream=None):
         assert isinstance(headers, HttpHeaders)
 
-        super(HttpMessageBase, self).__init__()
+        super().__init__()
         self._binding = binding
         self._headers = headers
 
@@ -332,7 +331,7 @@ class HttpMessageBase(NativeResource):
 
     @property
     def body_stream(self):
-        """InputStream: Stream of outgoing body."""
+        """InputStream: Binary stream of outgoing body."""
         return _awscrt.http_message_get_body_stream(self._binding)
 
     @body_stream.setter
@@ -352,7 +351,7 @@ class HttpRequest(HttpMessageBase):
         path (str): HTTP path-and-query value. Default value is "/".
         headers (Optional[HttpHeaders]): Optional headers. If None specified,
             an empty :class:`HttpHeaders` is created.
-        body_string(Optional[Union[InputStream, io.IOBase]]): Optional body as stream.
+        body_stream(Optional[Union[InputStream, io.IOBase]]): Optional body as binary stream.
     """
 
     __slots__ = ()
@@ -364,7 +363,7 @@ class HttpRequest(HttpMessageBase):
             headers = HttpHeaders()
 
         binding = _awscrt.http_message_new_request(headers)
-        super(HttpRequest, self).__init__(binding, headers, body_stream)
+        super().__init__(binding, headers, body_stream)
         self.method = method
         self.path = path
 
@@ -414,7 +413,7 @@ class HttpHeaders(NativeResource):
     __slots__ = ()
 
     def __init__(self, name_value_pairs=None):
-        super(HttpHeaders, self).__init__()
+        super().__init__()
         self._binding = _awscrt.http_headers_new()
         if name_value_pairs:
             self.add_pairs(name_value_pairs)
@@ -435,8 +434,8 @@ class HttpHeaders(NativeResource):
             name (str): Name.
             value (str): Value.
         """
-        assert isinstance_str(name)
-        assert isinstance_str(value)
+        assert isinstance(name, str)
+        assert isinstance(value, str)
         _awscrt.http_headers_add(self._binding, name, value)
 
     def add_pairs(self, name_value_pairs):
@@ -456,8 +455,8 @@ class HttpHeaders(NativeResource):
             name (str): Name.
             value (str): Value.
         """
-        assert isinstance_str(name)
-        assert isinstance_str(value)
+        assert isinstance(name, str)
+        assert isinstance(value, str)
         _awscrt.http_headers_set(self._binding, name, value)
 
     def get_values(self, name):
@@ -470,7 +469,7 @@ class HttpHeaders(NativeResource):
         Returns:
             Iterator[Tuple[str, str]]:
         """
-        assert isinstance_str(name)
+        assert isinstance(name, str)
         name = name.lower()
         for i in range(_awscrt.http_headers_count(self._binding)):
             name_i, value_i = _awscrt.http_headers_get_index(self._binding, i)
@@ -489,7 +488,7 @@ class HttpHeaders(NativeResource):
         Returns:
             str:
         """
-        assert isinstance_str(name)
+        assert isinstance(name, str)
         return _awscrt.http_headers_get(self._binding, name, default)
 
     def remove(self, name):
@@ -500,7 +499,7 @@ class HttpHeaders(NativeResource):
         Args:
             name (str): Header name.
         """
-        assert isinstance_str(name)
+        assert isinstance(name, str)
         _awscrt.http_headers_remove(self._binding, name)
 
     def remove_value(self, name, value):
@@ -512,8 +511,8 @@ class HttpHeaders(NativeResource):
             name (str): Name.
             value (str): Value.
         """
-        assert isinstance_str(name)
-        assert isinstance_str(value)
+        assert isinstance(name, str)
+        assert isinstance(value, str)
         _awscrt.http_headers_remove_value(self._binding, name, value)
 
     def clear(self):
@@ -542,7 +541,7 @@ class HttpProxyAuthenticationType(IntEnum):
     """Username and password"""
 
 
-class HttpProxyOptions(object):
+class HttpProxyOptions:
     """
     Proxy options for HTTP clients.
 
